#pragma rtGlobals=1		// Use modern global access method.

#include <BringDestToFront>

//functions are faster than macros
//functions can be inserted into the macros menu
//Proc procedures can use global wave names directly
//proc are macros that do not show up in the menu by default




//PSD function copied from wavemetrics
//inclusion of this function here should allow the PSD macro to
//play nice with IFDL; also added much higher resolution psds
//
// Given a long data wave create a short result wave containing the Power
// Spectral Density.  For the purposes of this macro, PSD is defined in
// terms of the power per frequency bin width. To get the total power you need
// to integrate. The signal is assumed to be a voltage measurement across a
// 1 ohm resistor.
// The name of the new wave is the name of the source + _psd
// Note: if you don't want the baggage of all the window functions, create your own version
// with your favorite and remove the window parameter (or choose a subset)
//
// Version 1.1, LH 971028
// Changes since 1.0: changed normalization to give results as defined above.
// Version 1.2, JP 010427
// Changes since 1.1: corrected winNorm for Hanning to 3/8.

Macro PSD(w,seglen,window)
	string w
	Prompt w "data wave:",popup WaveList("*",";","")
	variable seglen=1
	Prompt seglen,"segment length:",popup "256;512;1024;2048;4096;8192;16384;32768;65536"
	variable window=2
	Prompt window,"Window type:",popup "Square;Hann;Parzen;Welch;Hamming;"
						"BlackmanHarris3;KaiserBessel"
;
	PauseUpdate; silent 1
	
	variable npsd= 2^(7+seglen)				// number of points in group (resultant psd wave len= npsd/2+1)
	variable psdOffset= npsd/2					// offset each group by this amount
	variable psdFirst=0							// start of current group
	variable nsrc= numpnts($w)
	variable origlen = nsrc
	variable nsegs,winNorm						// count of number of segements and window normalization factor
	string destw=w+"_psd",srctmp=w+"_tmp"
	string winw=w+"_psdWin"					// window goes here
	
	if( npsd > nsrc/2)
		insertpoints nsrc, 2/(nsrc/(2^(7+seglen)))*nsrc-nsrc, $w
		variable newlen = numpnts($w)
		print "duplicating points for desired PSD resolution"
		print 2/(nsrc/(2^(7+seglen)))*nsrc
		variable c = 0
		variable p = nsrc
		do
			if(c == nsrc)
				c=0
			endif
			$w[p] = $w[c]
			c = c+1
			p = p+1
		while(p < newlen)
		nsrc= newlen
	endif
		
	if( npsd > nsrc/2 )
		print 2/(nsrc/(2^(7+seglen)))*nsrc
		Abort "psd: source wave should be MUCH longer than the segment length"		
	endif
	
	make/o/n=(npsd/2+1) $destw
	make/o/n=(npsd) $srctmp,$winw; $winw= 1
	if( window==1 )
		winNorm= 1
	else
		if( window==2 )
			Hanning $winw;winNorm=0.375				//  winNorm is avg squared value
		else
			if( window==3 )
				winNorm= Parzen($winw)
			else
				if( window==4 )
					winNorm= Welch($winw)
				else
					if( window==5 )
						winNorm= Hamming($winw)
					else
						if( window==6 )
							winNorm= BlackmanHarris3($winw)
						else
							if( window==7 )
								winNorm= KaiserBessel($winw)
							else
								Abort "unknown window index"
							endif
						endif
					endif
				endif
			endif
		endif
	endif	// (kinda makes you wish we had elseif or switch constructs, huh? - well we do but only in functions)

	Duplicate/O/R=[0,npsd-1] $w $srctmp; $srctmp *= $winw; fft $srctmp
	CopyScales/P $srctmp, $destw
	$destw= magsqr($srctmp)
	psdFirst= psdOffset
	nsegs=1
	do
		Duplicate/O/R=[psdFirst,psdFirst+npsd-1] $w $srctmp;   $srctmp *= $winw
		fft $srctmp;   $destw += magsqr($srctmp);   psdFirst += psdOffset; nsegs+=1
	while( psdFirst+npsd < nsrc )
	winNorm= 2*deltax($w)/(winNorm*nsegs*npsd);  $destw *= winNorm
	$destw[0] /= 2

	KillWaves $srctmp,$winw
	BringDestFront(destw)
	if( numpnts($destw) <= 129 )
		Modify mode($destw)=4,marker($destw)=19,msize($destw)=1
	else
		Modify mode($destw)=0
	endif
	
	Redimension/N=(origlen) $w
end
